@@ -29,7 +29,8 @@ class HTMLReportCreator:
         self.has_tags_page = config.do_process_tags()
         self._time_sampling_interval = "W"
         self._do_generate_index_page = False
-        self._do_plot_contribution_graph = False
+        self._is_blame_data_allowed = False
+        self._max_orphaned_extensions_count = 0
 
         templates_dir = os.path.join(HERE, self.templates_subdir)
         self.j2_env = Environment(loader=FileSystemLoader(templates_dir), trim_blocks=True)
@@ -48,14 +49,36 @@ class HTMLReportCreator:
         self._time_sampling_interval = offset
         return self
 
-    def plot_contribution_graph(self, do_plot=True):
-        self._do_plot_contribution_graph = do_plot
-        return self
+    def allow_blame_data(self):
+        self._is_blame_data_allowed = True
 
     def generate_index_page(self, do_generate: bool = True):
         self._do_generate_index_page = do_generate
         return self
 
+    def set_max_orphaned_extensions_count(self, count):
+        self._max_orphaned_extensions_count = count
+        return self
+
+    def _clamp_orphaned_extensions(self, extensions_df: pd.DataFrame, group_name: str = "~others~"):
+        # Group together all extensions used only once (probably not really extensions)
+        is_orphan = extensions_df["files_count"] <= self._max_orphaned_extensions_count
+        excluded = extensions_df[is_orphan]
+        print(excluded.shape)
+        if excluded.shape[0] > 0:
+            excluded_summary = excluded.agg({"size_bytes": ["sum"], "lines_count": ["sum", "count"]})
+            orphans_summary_df = pd.DataFrame(data=[{
+                "files_count": excluded_summary['lines_count']['count'],
+                "lines_count": excluded_summary['lines_count']['sum'],
+                "size_bytes": excluded_summary['size_bytes']['sum'].astype('int32')
+            }], index=[(False, group_name)]) # index is a tuple (is_binary, extension)
+
+            extensions_df = extensions_df[~is_orphan].sort_values(by="files_count", ascending=False)
+            # and we do not sort after we appended "orphans", as we want them to appear at the end
+            extensions_df = extensions_df.append(orphans_summary_df, sort=False)
+
+        return extensions_df
+
     def _get_recent_activity_data(self):
         recent_weekly_commits = self.git_repository_statistics.\
             get_recent_weekly_activity(self.recent_activity_period_weeks)
@@ -133,7 +156,6 @@ class HTMLReportCreator:
             copyfile(os.path.join(path, "general.html"), os.path.join(path, "index.html"))
 
     def make_general_page(self):
-        date_format_str = '%Y-%m-%d %H:%M'
         first_commit_datetime = datetime.datetime.fromtimestamp(self.git_repository_statistics.first_commit_timestamp)
         last_commit_datetime = datetime.datetime.fromtimestamp(self.git_repository_statistics.last_commit_timestamp)
 
@@ -149,12 +171,12 @@ class HTMLReportCreator:
             "total_lines_count": self.git_repository_statistics.total_lines_count,
             "added_lines_count": self.git_repository_statistics.total_lines_added,
             "removed_lines_count": self.git_repository_statistics.total_lines_removed,
-            "first_commit_date": first_commit_datetime.strftime(date_format_str),
-            "last_commit_date": last_commit_datetime.strftime(date_format_str),
+            "first_commit_date": first_commit_datetime,
+            "last_commit_date": last_commit_datetime,
         }
 
         generation_data = {
-            "datetime": datetime.datetime.today().strftime(date_format_str)
+            "datetime": datetime.datetime.today().strftime('%Y-%m-%d %H:%M')
         }
 
         page = HtmlPage(name="General",
@@ -246,9 +268,15 @@ class HTMLReportCreator:
             'authors_top': self.configuration['authors_top'],
             'total_commits_count': self.git_repository_statistics.total_commits_count,
             'total_lines_count': self.git_repository_statistics.total_lines_count,
-            'do_plot_contribution': self._do_plot_contribution_graph
+            'is_blame_data_available': self._is_blame_data_allowed,
         }
 
+        if self._is_blame_data_allowed:
+            project_data.update({
+                'top_knowledge_carriers': self.git_repository_statistics.head.get_top_knowledge_carriers()
+                    .head(self.configuration['authors_top'])
+            })
+
         raw_authors_data = self.git_repository_statistics.get_authors_ranking_by_month()
         ordered_months = raw_authors_data.index.get_level_values(0).unique().sort_values(ascending=False)
         project_data['months'] = []
@@ -273,7 +301,7 @@ class HTMLReportCreator:
                 'top_author': {'name': authors_in_year.index[0], 'commits_count': authors_in_year.iloc[0]},
                 'next_top_authors': ', '.join(list(authors_in_year.index[1:max_top_authors_index])),
                 'all_commits_count': authors_in_year.sum(),
-                'total_authors_count': authors_in_year.size
+                'total_authors_count': authors_in_year.size,
             })
 
         page = HtmlPage('Authors', project=project_data)
@@ -331,7 +359,7 @@ class HTMLReportCreator:
             "data": [{"key": domain, "y": commits_count} for domain, commits_count in email_domains_distribution.items()]
         }
 
-        if self._do_plot_contribution_graph:
+        if self._is_blame_data_allowed:
             # sort by contribution
             sorted_contribution = self.git_repository_statistics.head.authors_contribution.sort_values(ascending=False)
             # limit to only top authors
@@ -358,12 +386,25 @@ class HTMLReportCreator:
         return authors_plot
 
     def make_files_page(self):
+        file_ext_summary = self.git_repository_statistics.head.files_extensions_summary
+        if self._max_orphaned_extensions_count > 0:
+            file_ext_summary = self._clamp_orphaned_extensions(file_ext_summary)
+        else:
+            file_ext_summary = file_ext_summary.sort_values(by="files_count", ascending=False)
+
         project_data = {
             'total_files_count': self.git_repository_statistics.head.files_count,
             'total_lines_count': self.git_repository_statistics.total_lines_count,
             'size': self.git_repository_statistics.head.size,
-            'file_summary': self.git_repository_statistics.head.files_extensions_summary
+            'file_summary': file_ext_summary,
+            'is_blame_data_available': self._is_blame_data_allowed
         }
+        if self._is_blame_data_allowed:
+            project_data.update({
+                'top_files_by_contributors_count': self.git_repository_statistics.head.get_top_files_by_contributors_count(),
+                'monoauthor_files_count': self.git_repository_statistics.head.monoauthor_files.count(),
+                'lost_knowledge_ratio': self.git_repository_statistics.head.get_lost_knowledge_percentage()
+            })
 
         page = HtmlPage('Files', project=project_data)
         page.add_plot(self.make_files_plot())
